/**
 * Den Interrupt- und Scrollmist habe ich rausgelscht.
 * \todo Extrazeichenansteuerung rausfinden
 */

#include <avr/io.h>
#include <avr/pgmspace.h>
#include <inttypes.h>
#include <avr/signal.h>
#include "main.h"
#include "bf_lcd.h"
#include "iom169missing.h"

#define BOOL    char
#define FALSE   0
#define TRUE    (!FALSE)


unsigned int lcd_character_table[] PROGMEM =
{
    0x0A51,     // '*' (?)
    0x2A80,     // '+'
    0x0000,     // ',' (Not defined)
    0x0A00,     // '-'
    0x0A51,     // '.' Degree sign
    0x0001,     // '/' (Not defined)
    0x5559,     // '0'
    0x0118,     // '1'
    0x1e11,     // '2
    0x1b11,     // '3
    0x0b50,     // '4
    0x1b41,     // '5
    0x1f41,     // '6
    0x0111,     // '7
    0x1f51,     // '8
    0x1b51,     // '9'
    0x0000,     // ':' (Not defined)
    0x0000,     // ';' (Not defined)
    0x0000,     // '<' (Not defined)
    0x0000,     // '=' (Not defined)
    0x0000,     // '>' (Not defined)
    0x0000,     // '?' (Not defined)
    0x0000,     // '@' (Not defined)
    0x0f51,     // 'A' (+ 'a')
    0x3991,     // 'B' (+ 'b')
    0x1441,     // 'C' (+ 'c')
    0x3191,     // 'D' (+ 'd')
    0x1e41,     // 'E' (+ 'e')
    0x0e41,     // 'F' (+ 'f')
    0x1d41,     // 'G' (+ 'g')
    0x0f50,     // 'H' (+ 'h')
    0x2080,     // 'I' (+ 'i')
    0x1510,     // 'J' (+ 'j')
    0x8648,     // 'K' (+ 'k')
    0x1440,     // 'L' (+ 'l')
    0x0578,     // 'M' (+ 'm')
    0x8570,     // 'N' (+ 'n')
    0x1551,     // 'O' (+ 'o')
    0x0e51,     // 'P' (+ 'p')
    0x9551,     // 'Q' (+ 'q')
    0x8e51,     // 'R' (+ 'r')
    0x9021,     // 'S' (+ 's')
    0x2081,     // 'T' (+ 't')
    0x1550,     // 'U' (+ 'u')
    0x4448,     // 'V' (+ 'v')
    0xc550,     // 'W' (+ 'w')
    0xc028,     // 'X' (+ 'x')
    0x2028,     // 'Y' (+ 'y')
    0x5009,     // 'Z' (+ 'z')
    0x0000,     // '[' (Not defined)
    0x0000,     // '\' (Not defined)
    0x0000,     // ']' (Not defined)
    0x0000,     // '^' (Not defined)
    0x0000      // '_'
};


/*****************************************************************************
*  Initialize LCD_displayData buffer.
*  Set up the LCD (timing, contrast, etc.)
*****************************************************************************/
void lcd_init (void)
{
    lcd_allsegments(FALSE);                     // Clear segment buffer.

    LCD_CONTRAST_LEVEL(LCD_INITIAL_CONTRAST);    //Set the LCD contrast level

    // Select asynchronous clock source, enable all COM pins and enable all
    // segment pins.
    LCDCRB = (1<<LCDCS) | (3<<LCDMUX0) | (7<<LCDPM0);

    // Set LCD prescaler to give a framerate of 32,0 Hz
    LCDFRR = (0<<LCDPS0) | (7<<LCDCD0);    

    LCDCRA = (1<<LCDEN) | (1<<LCDAB);           // Enable LCD and set low power waveform

}


/**
* \param char c:	The symbol to be displayed in a LCD digit
* \param char digit: In which digit (0-5) the symbol should be displayed
* Note: Digit 0 is the first used digit on the LCD,
* Beschreibt direkt die LCD-Register
**/
void lcd_writedigit(char c, char digit)
{

    unsigned int seg = 0x0000;                  // Holds the segment pattern
    char mask, nibble;
    char *ptr;
    char i;


    if (digit > 5)                              // Skip if digit is illegal
        return;

    //Lookup character table for segmet data
    if ((c >= '*') && (c <= 'z'))
    {
        // c is a letter
        if (c >= 'a')                           // Convert to upper case
            c &= ~0x20;                         // if necessarry

        c -= '*';

		seg = (unsigned int) pgm_read_word(&lcd_character_table[(uint8_t)c]); 
	}

    // Adjust mask according to LCD segment mapping
    if (digit & 0x01)
        mask = 0x0F;                // Digit 1, 3, 5
    else
        mask = 0xF0;                // Digit 0, 2, 4

    ptr = pLCDREG + (digit >> 1);  // digit = {0,0,1,1,2,2}

    for (i = 0; i < 4; i++)
    {
        nibble = seg & 0x000F;
        seg >>= 4;
        if (digit & 0x01)
            nibble <<= 4;
        *ptr = (*ptr & mask) | nibble;
        ptr += 5;
    }
}

/**
 * \param character
 * \return 0
 */
int lcd_writechar(char c)
{
	static uint8_t lcdpos;
	static struct{
		int colon:1;
	}flag;
		
	switch(c)
	{
		case ':':	*(pLCDREG+8)=0x01; 
				//lcdpos=(lcdpos&~1)+2;f
				flag.colon=1;
				return 0;
			
		case '\x02':	*(pLCDREG+8)|=0x03;return 0;
		case '':	lcd_writedigit('.',lcdpos); break;
		case '\r':	
		case '\n':	//while(lcdpos < 6)
				//	lcd_writedigit(' ',lcdpos++);
				lcdpos=0; 
				if (!flag.colon) 
					*(pLCDREG+8)&=~0x01;	
				flag.colon=0;
				return 0;
		case '\t':	lcdpos=(lcdpos&~1)+2; return 0;	//after next colon
		default:	
				lcd_writedigit(c,lcdpos);
				break;
	}

	lcdpos++;
	lcdpos=(lcdpos>5)?0:lcdpos;

	return 0;
}

/**
* \param  show -  [TRUE;FALSE]
*
*   Purpose :       shows or hide all all LCD segments on the LCD
*
*/
void lcd_allsegments(char show)
{
    unsigned char i;
    char * ptr;


     ptr = pLCDREG;

    for (i = 0; i < 4; i++)
    {
	    if (show)
		*ptr |= 0xff;
	    else	
		*ptr &= 0;    	
        ptr += 5;
    }
}

void lcd_off()
{
	loop_until_bit_is_set(LCDCRA,LCDIF)
		;
	LCDCRA = _BV(LCDEN)|_BV(LCDIF)|_BV(LCDBL);
	loop_until_bit_is_set(LCDCRA,LCDIF)
		;
	LCDCRA&=~_BV(LCDEN);
	PRR |=_BV(PRLCD);
}
