#include <stdio.h>
#include <avr/io.h>
#include <stdlib.h>
#include <avr/pgmspace.h>

#include "settings.h"
#include "dataflash.h"

extern FILE *uart;

void df_spi_init (void)
{
	PORTB |= _BV(PB3)|_BV(PB2)|_BV(PB1)|_BV(PB0);
        DDRB |= _BV(DDB2)|_BV(DDB1)|_BV(DDB0);
	SPSR = _BV(SPI2X);                                      //SPI double speed settings       
	SPCR = _BV(SPE)|_BV(MSTR)|_BV(CPHA)|_BV(CPOL);    //Enable SPI in Master mode, mode 3, Fosc/4                                                                        
}

uint8_t df_spi_rw(uint8_t out)
{
	uint8_t in;
	SPDR=out;
	loop_until_bit_is_set(SPSR,SPIF)
		;
	in=SPDR;
	return in;
}

void cont_array_read_init()
{
	df_cs_inactive;
	df_cs_active;
	df_spi_rw(0x68);
	df_spi_rw(0);
	df_spi_rw(0);
	df_spi_rw(0);
	df_spi_rw(0);
	df_spi_rw(0);
	df_spi_rw(0);
	df_spi_rw(0);
}

void buffer_write_init(uint8_t bufferno, uint16_t adr)
{
	df_cs_inactive;
	df_cs_active;
	df_spi_rw( (bufferno==1) ? 0x84 : 0x87);
	df_spi_rw( (uint16_t) (adr>>8) );
	df_spi_rw( adr );
}

void buffer_read_init(uint8_t bufferno, uint16_t adr)
{
	df_cs_inactive;
	df_cs_active;
	df_spi_rw( (bufferno==1) ? 0x54 : 0x56);
	df_spi_rw(0);
	df_spi_rw( (uint16_t) (adr>>8) );
	df_spi_rw( adr );
}

void buffer_to_flash_no_erase(uint8_t bufferno, uint16_t pageadr)
{
	df_cs_active;
        df_spi_rw( (bufferno==1) ? 0x88 : 0x89 );
        df_spi_rw( (uint16_t) pageadr>>7);
	df_spi_rw( (uint16_t) pageadr<<1);
        df_spi_rw(0);
        df_cs_inactive;
}
					

void block_erase(uint8_t blockadr)
{
	df_cs_inactive;
	df_cs_active;
	df_spi_rw(0x50);
	df_spi_rw(blockadr>>4);
	df_spi_rw(blockadr<<4);
	df_spi_rw(0);
}

void erase_all()
{
	int i;
#if DEBUG
	fprintf(uart,"%s\n",__FUNCTION__);
#endif
	for(i=0;i<256;i++)
	{
		block_erase(i);
#if DEBUG
		fprintf(uart,"Deleting Page %2.2X\n",i);
#endif
		printf_P(PSTR("Del %2.2X\n"),i);
		while(!(flash_status() & 0x80))
			;
	}
}

uint8_t flash_status()
{
	uint8_t status;
        df_cs_inactive;
        df_cs_active;
        df_spi_rw(StatusRegRead);       
        status=df_spi_rw(0);
        df_cs_inactive;
}					
	
